@@ -195,45 +195,56 @@ impl MusicalTime {
         Seconds(self.as_fractional_beats() * 60.0 / bpm)
     }
 
-    /// Convert to the corresponding discrete [`SampleTime`]. This will be rounded to the nearest sample.
+    /// Convert to the corresponding discrete [`RealSampleTime`]. This will be rounded to the nearest sample.
     ///
     /// Note that this must be re-calculated after recieving a new [`SampleRate`].
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     #[inline]
-    pub fn to_nearest_sample_round(&self, bpm: f64, sample_rate: SampleRate) -> SampleTime {
-        self.to_seconds(bpm).to_nearest_sample_round(sample_rate)
+    pub fn to_nearest_real_sample_round(
+        &self,
+        bpm: f64,
+        sample_rate: SampleRate,
+    ) -> RealSampleTime {
+        self.to_seconds(bpm)
+            .to_nearest_real_sample_round(sample_rate)
     }
 
-    /// Convert to the corresponding discrete [`SampleTime`]. This will be floored to the nearest sample.
+    /// Convert to the corresponding discrete [`RealSampleTime`]. This will be floored to the nearest sample.
     ///
     /// Note that this must be re-calculated after recieving a new [`SampleRate`].
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     #[inline]
-    pub fn to_nearest_sample_floor(&self, bpm: f64, sample_rate: SampleRate) -> SampleTime {
-        self.to_seconds(bpm).to_nearest_sample_floor(sample_rate)
+    pub fn to_nearest_real_sample_floor(
+        &self,
+        bpm: f64,
+        sample_rate: SampleRate,
+    ) -> RealSampleTime {
+        self.to_seconds(bpm)
+            .to_nearest_real_sample_floor(sample_rate)
     }
 
-    /// Convert to the corresponding discrete [`SampleTime`]. This will be ceil-ed to the nearest sample.
+    /// Convert to the corresponding discrete [`RealSampleTime`]. This will be ceil-ed to the nearest sample.
     ///
     /// Note that this must be re-calculated after recieving a new [`SampleRate`].
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     #[inline]
-    pub fn to_nearest_sample_ceil(&self, bpm: f64, sample_rate: SampleRate) -> SampleTime {
-        self.to_seconds(bpm).to_nearest_sample_ceil(sample_rate)
+    pub fn to_nearest_real_sample_ceil(&self, bpm: f64, sample_rate: SampleRate) -> RealSampleTime {
+        self.to_seconds(bpm)
+            .to_nearest_real_sample_ceil(sample_rate)
     }
 
-    /// Convert to the corresponding discrete [`SampleTime`] floored to the nearest sample,
+    /// Convert to the corresponding discrete [`RealSampleTime`] floored to the nearest sample,
     /// while also returning the fractional sub-sample part.
     ///
     /// Note that this must be re-calculated after recieving a new [`SampleRate`].
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     #[inline]
-    pub fn to_sub_sample(&self, bpm: f64, sample_rate: SampleRate) -> (SampleTime, f64) {
-        self.to_seconds(bpm).to_sub_sample(sample_rate)
+    pub fn to_sub_real_sample(&self, bpm: f64, sample_rate: SampleRate) -> (RealSampleTime, f64) {
+        self.to_seconds(bpm).to_sub_real_sample(sample_rate)
     }
 
     /// Convert to the corresponding discrete [`SuperSampleTime`]. This will be rounded to the nearest super-sample.
@@ -368,12 +379,12 @@ impl Seconds {
         self.0 as f32
     }
 
-    /// Creates a new time in `Seconds` from [`SampleTime`] and a [`SampleRate`].
+    /// Creates a new time in `Seconds` from [`RealSampleTime`] and a [`SampleRate`].
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     /// [`SampleRate`]: struct.SampleRate.html
     #[inline]
-    pub fn from_sample(sample_time: SampleTime, sample_rate: SampleRate) -> Self {
+    pub fn from_real_sample(sample_time: RealSampleTime, sample_rate: SampleRate) -> Self {
         sample_time.to_seconds(sample_rate)
     }
 
@@ -385,78 +396,78 @@ impl Seconds {
         super_sample_time.to_seconds()
     }
 
-    /// Convert to discrete [`SampleTime`] with the given [`SampleRate`]. This will
+    /// Convert to discrete [`RealSampleTime`] with the given [`SampleRate`]. This will
     /// be rounded to the nearest sample.
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     /// [`SampleRate`]: struct.SampleRate.html
     #[inline]
-    pub fn to_nearest_sample_round(&self, sample_rate: SampleRate) -> SampleTime {
-        SampleTime((self.0 * sample_rate).round() as i64)
+    pub fn to_nearest_real_sample_round(&self, sample_rate: SampleRate) -> RealSampleTime {
+        RealSampleTime((self.0 * sample_rate).round() as i64)
     }
 
-    /// Convert to discrete [`SampleTime`] with the given [`SampleRate`]. This will
+    /// Convert to discrete [`RealSampleTime`] with the given [`SampleRate`]. This will
     /// be floored to the nearest sample.
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     /// [`SampleRate`]: struct.SampleRate.html
     #[inline]
-    pub fn to_nearest_sample_floor(&self, sample_rate: SampleRate) -> SampleTime {
-        SampleTime((self.0 * sample_rate).floor() as i64)
+    pub fn to_nearest_real_sample_floor(&self, sample_rate: SampleRate) -> RealSampleTime {
+        RealSampleTime((self.0 * sample_rate).floor() as i64)
     }
 
-    /// Convert to discrete [`SampleTime`] with the given [`SampleRate`]. This will
+    /// Convert to discrete [`RealSampleTime`] with the given [`SampleRate`]. This will
     /// be ceil-ed to the nearest sample.
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     /// [`SampleRate`]: struct.SampleRate.html
     #[inline]
-    pub fn to_nearest_sample_ceil(&self, sample_rate: SampleRate) -> SampleTime {
-        SampleTime((self.0 * sample_rate).ceil() as i64)
+    pub fn to_nearest_real_sample_ceil(&self, sample_rate: SampleRate) -> RealSampleTime {
+        RealSampleTime((self.0 * sample_rate).ceil() as i64)
     }
 
-    /// Convert to discrete [`SampleTime`] given the [`SampleRate`] floored to the nearest
+    /// Convert to discrete [`RealSampleTime`] given the [`SampleRate`] floored to the nearest
     /// sample, while also return the fractional sub-sample part.
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     /// [`SampleRate`]: struct.SampleRate.html
     #[inline]
-    pub fn to_sub_sample(&self, sample_rate: SampleRate) -> (SampleTime, f64) {
+    pub fn to_sub_real_sample(&self, sample_rate: SampleRate) -> (RealSampleTime, f64) {
         let smps_f64 = self.0 * sample_rate;
-        (SampleTime(smps_f64.floor() as i64), smps_f64.fract())
+        (RealSampleTime(smps_f64.floor() as i64), smps_f64.fract())
     }
 
     /// Convert to discrete [`SuperSampleTime`]. This will
     /// be rounded to the nearest super-sample.
     ///
-    /// [`SuperSampleTime`]: struct.SampleTime.html
+    /// [`SuperSampleTime`]: struct.RealSampleTime.html
     #[inline]
     pub fn to_nearest_super_sample_round(&self) -> SuperSampleTime {
         SuperSampleTime((self.0 * 28_224_000.0).round() as i64)
     }
 
-    /// Convert to discrete [`SampleTime`]. This will
+    /// Convert to discrete [`RealSampleTime`]. This will
     /// be floored to the nearest super-sample.
     ///
-    /// [`SuperSampleTime`]: struct.SampleTime.html
+    /// [`SuperSampleTime`]: struct.RealSampleTime.html
     #[inline]
     pub fn to_nearest_super_sample_floor(&self) -> SuperSampleTime {
         SuperSampleTime((self.0 * 28_224_000.0).floor() as i64)
     }
 
-    /// Convert to discrete [`SampleTime`]. This will
+    /// Convert to discrete [`RealSampleTime`]. This will
     /// be ceil-ed to the nearest super-sample.
     ///
-    /// [`SuperSampleTime`]: struct.SampleTime.html
+    /// [`SuperSampleTime`]: struct.RealSampleTime.html
     #[inline]
     pub fn to_nearest_super_sample_ceil(&self) -> SuperSampleTime {
         SuperSampleTime((self.0 * 28_224_000.0).ceil() as i64)
     }
 
-    /// Convert to discrete [`SampleTime`] floored to the nearest
+    /// Convert to discrete [`RealSampleTime`] floored to the nearest
     /// super-sample, while also return the fractional sub-super-sample part.
     ///
-    /// [`SuperSampleTime`]: struct.SampleTime.html
+    /// [`SuperSampleTime`]: struct.RealSampleTime.html
     #[inline]
     pub fn to_sub_super_sample(&self) -> (SuperSampleTime, f64) {
         let super_smps_f64 = self.0 * 28_224_000.0;
@@ -565,9 +576,9 @@ impl DivAssign<Seconds> for Seconds {
 
 /// Unit of time in discrete samples.
 #[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Eq, Hash)]
-pub struct SampleTime(pub i64);
+pub struct RealSampleTime(pub i64);
 
-impl SampleTime {
+impl RealSampleTime {
     #[inline]
     pub fn new(samples: i64) -> Self {
         Self(samples)
@@ -620,71 +631,71 @@ impl SampleTime {
     /// [`SampleRate`]: struct.SampleRate.html
     #[inline]
     pub fn to_super_sample(&self, sample_rate: SampleRate) -> SuperSampleTime {
-        SuperSampleTime::from_sample_time(*self, sample_rate)
+        SuperSampleTime::from_real_sample_time(*self, sample_rate)
     }
 }
 
-impl Default for SampleTime {
+impl Default for RealSampleTime {
     fn default() -> Self {
-        SampleTime(0)
+        RealSampleTime(0)
     }
 }
 
-impl From<i8> for SampleTime {
+impl From<i8> for RealSampleTime {
     fn from(s: i8) -> Self {
-        SampleTime(i64::from(s))
+        RealSampleTime(i64::from(s))
     }
 }
-impl From<u8> for SampleTime {
+impl From<u8> for RealSampleTime {
     fn from(s: u8) -> Self {
-        SampleTime(i64::from(s))
+        RealSampleTime(i64::from(s))
     }
 }
-impl From<i16> for SampleTime {
+impl From<i16> for RealSampleTime {
     fn from(s: i16) -> Self {
-        SampleTime(i64::from(s))
+        RealSampleTime(i64::from(s))
     }
 }
-impl From<u16> for SampleTime {
+impl From<u16> for RealSampleTime {
     fn from(s: u16) -> Self {
-        SampleTime(i64::from(s))
+        RealSampleTime(i64::from(s))
     }
 }
-impl From<i32> for SampleTime {
+impl From<i32> for RealSampleTime {
     fn from(s: i32) -> Self {
-        SampleTime(i64::from(s))
+        RealSampleTime(i64::from(s))
     }
 }
-impl From<u32> for SampleTime {
+impl From<u32> for RealSampleTime {
     fn from(s: u32) -> Self {
-        SampleTime(i64::from(s))
+        RealSampleTime(i64::from(s))
     }
 }
-impl From<i64> for SampleTime {
+impl From<i64> for RealSampleTime {
     fn from(s: i64) -> Self {
-        SampleTime(s)
+        RealSampleTime(s)
     }
 }
 
-impl Add<SampleTime> for SampleTime {
+impl Add<RealSampleTime> for RealSampleTime {
     type Output = Self;
     fn add(self, rhs: Self) -> Self::Output {
         Self(self.0 + rhs.0)
     }
 }
-impl Sub<SampleTime> for SampleTime {
+impl Sub<RealSampleTime> for RealSampleTime {
     type Output = Self;
     fn sub(self, rhs: Self) -> Self::Output {
         Self(self.0 - rhs.0)
     }
 }
 
-impl AddAssign<SampleTime> for SampleTime {
+impl AddAssign<RealSampleTime> for RealSampleTime {
     fn add_assign(&mut self, other: Self) {
         self.0 += other.0;
     }
 }
-impl SubAssign<SampleTime> for SampleTime {
+impl SubAssign<RealSampleTime> for RealSampleTime {
     fn sub_assign(&mut self, other: Self) {
         self.0 -= other.0;
     }
@@ -712,7 +723,7 @@ impl SuperSampleTime {
         Self(super_samples as i64)
     }
 
-    pub fn from_sample_time(sample_time: SampleTime, sample_rate: SampleRate) -> Self {
+    pub fn from_real_sample_time(sample_time: RealSampleTime, sample_rate: SampleRate) -> Self {
         match sample_rate.0 as usize {
             44100 => Self(sample_time.0 * (28_224_000 / 44100)),
             48000 => Self(sample_time.0 * (28_224_000 / 48000)),
@@ -754,13 +765,13 @@ impl SuperSampleTime {
         self.to_seconds().to_musical(bpm)
     }
 
-    /// Convert to the corresponding [`SampleTime`] from the given [`SampleRate`].
+    /// Convert to the corresponding [`RealSampleTime`] from the given [`SampleRate`].
     ///
-    /// [`SampleTime`]: struct.SampleTime.html
+    /// [`RealSampleTime`]: struct.RealSampleTime.html
     /// [`SampleRate`]: struct.SampleRate.html
     #[inline]
-    pub fn to_sample_time(&self, sample_rate: SampleRate) -> SampleTime {
-        SampleTime((self.0 as f64 * sample_rate.0 / 28_224_000.0).round() as i64)
+    pub fn to_real_sample_time(&self, sample_rate: SampleRate) -> RealSampleTime {
+        RealSampleTime((self.0 as f64 * sample_rate.0 / 28_224_000.0).round() as i64)
     }
 }
 
@@ -829,3 +840,253 @@ impl SubAssign<SuperSampleTime> for SuperSampleTime {
         self.0 -= other.0;
     }
 }
+
+/// Unit of time length (of a single de-interleaved channel) in real samples. This is similar
+/// to [`RealSampleTime`] except this will always be positive.
+#[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Eq, Hash)]
+pub struct RealFrames(pub usize);
+
+impl RealFrames {
+    #[inline]
+    pub fn new(real_frames: usize) -> Self {
+        Self(real_frames)
+    }
+
+    /// Convert to the corresponding time in [`Seconds`] with the given [`SampleRate`].
+    ///
+    /// [`Seconds`]: struct.Seconds.html
+    /// [`SampleRate`]: struct.SampleRate.html
+    #[inline]
+    pub fn to_seconds(&self, sample_rate: SampleRate) -> Seconds {
+        Seconds(self.0 as f64 / sample_rate)
+    }
+
+    /// Convert to the corresponding [`MusicalTime`].
+    ///
+    /// Note that this must be re-calculated after recieving a new [`SampleRate`].
+    ///
+    /// [`MusicalTime`]: struct.MusicalTime.html
+    #[inline]
+    pub fn to_musical(&self, bpm: f64, sample_rate: SampleRate) -> MusicalTime {
+        self.to_seconds(sample_rate).to_musical(bpm)
+    }
+
+    /// Convert to the corresponding [`RealSampleTime`].
+    ///
+    /// [`RealSampleTimes`]: struct.RealSampleTime.html
+    #[inline]
+    pub fn to_real_sample_time(&self) -> RealSampleTime {
+        RealSampleTime::new(self.0 as i64)
+    }
+
+    /// Convert to the corresponding time length in [`SuperFrames`] from the given [`SampleRate`].
+    ///
+    /// [`SuperFrames`]: struct.SuperFrames.html
+    /// [`SampleRate`]: struct.SampleRate.html
+    #[inline]
+    pub fn to_super_frames(&self, sample_rate: SampleRate) -> SuperFrames {
+        SuperFrames::from_real_frames(*self, sample_rate)
+    }
+}
+
+impl Default for RealFrames {
+    fn default() -> Self {
+        RealFrames(0)
+    }
+}
+
+impl From<u8> for RealFrames {
+    fn from(s: u8) -> Self {
+        RealFrames(usize::from(s))
+    }
+}
+impl From<u16> for RealFrames {
+    fn from(s: u16) -> Self {
+        RealFrames(usize::from(s))
+    }
+}
+impl From<usize> for RealFrames {
+    fn from(s: usize) -> Self {
+        RealFrames(s)
+    }
+}
+
+impl Add<RealFrames> for RealFrames {
+    type Output = Self;
+    fn add(self, rhs: Self) -> Self::Output {
+        Self(self.0 + rhs.0)
+    }
+}
+impl Sub<RealFrames> for RealFrames {
+    type Output = Self;
+    fn sub(self, rhs: Self) -> Self::Output {
+        Self(self.0 - rhs.0)
+    }
+}
+
+impl Add<RealFrames> for RealSampleTime {
+    type Output = Self;
+    fn add(self, rhs: RealFrames) -> Self::Output {
+        Self(self.0 + rhs.0 as i64)
+    }
+}
+impl Sub<RealFrames> for RealSampleTime {
+    type Output = Self;
+    fn sub(self, rhs: RealFrames) -> Self::Output {
+        Self(self.0 - rhs.0 as i64)
+    }
+}
+
+impl AddAssign<RealFrames> for RealFrames {
+    fn add_assign(&mut self, other: Self) {
+        self.0 += other.0;
+    }
+}
+impl SubAssign<RealFrames> for RealFrames {
+    fn sub_assign(&mut self, other: Self) {
+        self.0 -= other.0;
+    }
+}
+
+impl AddAssign<RealFrames> for RealSampleTime {
+    fn add_assign(&mut self, other: RealFrames) {
+        self.0 += other.0 as i64;
+    }
+}
+impl SubAssign<RealFrames> for RealSampleTime {
+    fn sub_assign(&mut self, other: RealFrames) {
+        self.0 -= other.0 as i64;
+    }
+}
+
+/// Unit of time length (of a single de-interleaved channel) in super-samples. This is similar
+/// to [`SuperSampleTime`] except this will always be positive.
+#[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Eq, Hash)]
+pub struct SuperFrames(pub usize);
+
+impl SuperFrames {
+    #[inline]
+    pub fn new(real_frames: usize) -> Self {
+        Self(real_frames)
+    }
+
+    pub fn from_real_frames(real_frames: RealFrames, sample_rate: SampleRate) -> Self {
+        match sample_rate.0 as usize {
+            44100 => Self(real_frames.0 * (28_224_000 / 44100)),
+            48000 => Self(real_frames.0 * (28_224_000 / 48000)),
+            88200 => Self(real_frames.0 * (28_224_000 / 88200)),
+            96000 => Self(real_frames.0 * (28_224_000 / 96000)),
+            176400 => Self(real_frames.0 * (28_224_000 / 176400)),
+            192000 => Self(real_frames.0 * (28_224_000 / 192000)),
+            22050 => Self(real_frames.0 * (28_224_000 / 22050)),
+            24000 => Self(real_frames.0 * (28_224_000 / 24000)),
+            _ => Self((real_frames.0 as f64 * (28_224_000.0 / sample_rate.0)).round() as usize),
+        }
+    }
+
+    /// Convert to the corresponding time in [`Seconds`] with the given [`SampleRate`].
+    ///
+    /// [`Seconds`]: struct.Seconds.html
+    /// [`SampleRate`]: struct.SampleRate.html
+    #[inline]
+    pub fn to_seconds(&self, sample_rate: SampleRate) -> Seconds {
+        Seconds(self.0 as f64 / sample_rate)
+    }
+
+    /// Convert to the corresponding [`MusicalTime`].
+    ///
+    /// Note that this must be re-calculated after recieving a new [`SampleRate`].
+    ///
+    /// [`MusicalTime`]: struct.MusicalTime.html
+    #[inline]
+    pub fn to_musical(&self, bpm: f64, sample_rate: SampleRate) -> MusicalTime {
+        self.to_seconds(sample_rate).to_musical(bpm)
+    }
+
+    /// Convert to the corresponding [`SuperSampleTime`].
+    ///
+    /// [`SuperSampleTimes`]: struct.SuperSampleTime.html
+    #[inline]
+    pub fn to_super_sample_time(&self) -> SuperSampleTime {
+        SuperSampleTime::new(self.0 as i64)
+    }
+
+    /// Convert to the corresponding time length in [`RealFrames`] from the given [`SampleRate`].
+    ///
+    /// [`RealFrames`]: struct.RealFrames.html
+    /// [`SampleRate`]: struct.SampleRate.html
+    #[inline]
+    pub fn to_real_frames(&self, sample_rate: SampleRate) -> RealFrames {
+        RealFrames((self.0 as f64 * sample_rate.0 / 28_224_000.0).round() as usize)
+    }
+}
+
+impl Default for SuperFrames {
+    fn default() -> Self {
+        SuperFrames(0)
+    }
+}
+
+impl From<u8> for SuperFrames {
+    fn from(s: u8) -> Self {
+        SuperFrames(usize::from(s))
+    }
+}
+impl From<u16> for SuperFrames {
+    fn from(s: u16) -> Self {
+        SuperFrames(usize::from(s))
+    }
+}
+impl From<usize> for SuperFrames {
+    fn from(s: usize) -> Self {
+        SuperFrames(s)
+    }
+}
+
+impl Add<SuperFrames> for SuperFrames {
+    type Output = Self;
+    fn add(self, rhs: Self) -> Self::Output {
+        Self(self.0 + rhs.0)
+    }
+}
+impl Sub<SuperFrames> for SuperFrames {
+    type Output = Self;
+    fn sub(self, rhs: Self) -> Self::Output {
+        Self(self.0 - rhs.0)
+    }
+}
+
+impl Add<SuperFrames> for SuperSampleTime {
+    type Output = Self;
+    fn add(self, rhs: SuperFrames) -> Self::Output {
+        Self(self.0 + rhs.0 as i64)
+    }
+}
+impl Sub<SuperFrames> for SuperSampleTime {
+    type Output = Self;
+    fn sub(self, rhs: SuperFrames) -> Self::Output {
+        Self(self.0 - rhs.0 as i64)
+    }
+}
+
+impl AddAssign<SuperFrames> for SuperFrames {
+    fn add_assign(&mut self, other: Self) {
+        self.0 += other.0;
+    }
+}
+impl SubAssign<SuperFrames> for SuperFrames {
+    fn sub_assign(&mut self, other: Self) {
+        self.0 -= other.0;
+    }
+}
+
+impl AddAssign<SuperFrames> for SuperSampleTime {
+    fn add_assign(&mut self, other: SuperFrames) {
+        self.0 += other.0 as i64;
+    }
+}
+impl SubAssign<SuperFrames> for SuperSampleTime {
+    fn sub_assign(&mut self, other: SuperFrames) {
+        self.0 -= other.0 as i64;
+    }
+}
